@@ -7,19 +7,20 @@ import signal
 import sys
 import time
 from pathlib import Path
+from threading import Event
 
 import click
 import psutil
 from flask import Config
 
 from ..config import DEFAULT_CONFIG
-from ..process_controller.node_controller import find_node_executable
 from ..process_controller.elementsd_controller import ElementsPlainController
+from ..process_controller.node_controller import find_node_executable
 from .utils import (
     Echo,
+    compute_data_dir_and_set_config_obj,
     kill_node_process,
     purge_node_data_dir,
-    compute_data_dir_and_set_config_obj,
 )
 
 logger = logging.getLogger(__name__)
@@ -42,6 +43,17 @@ logger = logging.getLogger(__name__)
     "--data-dir",
     help="Specify a (maybe not yet existing) datadir. Works only with --nodocker. (Default is /tmp/bitcoind_plain_datadir)",
 )
+@click.option(
+    "--port",
+    default=18443,
+    help="Specify a port the Bitcoind should run on. Default is 18443.",
+)
+@click.option(
+    "--log-stdout",
+    is_flag=True,
+    default=False,
+    help="Will spit out logs on stdout if set",
+)
 @click.option(
     "--mining/--no-mining",
     default=True,
@@ -79,6 +91,8 @@ def bitcoind(
     nodocker,
     docker_tag,
     data_dir,
+    port,
+    log_stdout,
     mining,
     mining_period,
     reset,
@@ -96,6 +110,8 @@ def bitcoind(
         nodocker,
         docker_tag,
         data_dir,
+        port,
+        log_stdout,
         mining,
         mining_period,
         reset,
@@ -114,6 +130,17 @@ def bitcoind(
     "--data-dir",
     help="Specify a (maybe not yet existing) datadir. Works only with --nodocker. (Default is /tmp/bitcoind_plain_datadir)",
 )
+@click.option(
+    "--port",
+    default=18884,
+    help="Specify a port the elementsd should run on. Default is 18884.",
+)
+@click.option(
+    "--log-stdout",
+    is_flag=True,
+    default=False,
+    help="Will spit out logs on stdout if set",
+)
 @click.option(
     "--mining/--no-mining",
     default=True,
@@ -149,6 +176,8 @@ def bitcoind(
 def elementsd(
     quiet,
     data_dir,
+    port,
+    log_stdout,
     mining,
     mining_period,
     reset,
@@ -156,7 +185,7 @@ def elementsd(
     cleanuphard,
     config,
 ):
-    """This will start a bitcoind regtest and mines a block every mining-period.
+    """This will start a elements regtest and mines a block every mining-period.
     If a bitcoind is already running on port 18443, it won't start another one. If you CTRL-C this, the bitcoind will
     still continue to run. You have to shut it down.
     """
@@ -166,6 +195,8 @@ def elementsd(
         True,  # nodocker
         None,  # docker_tag
         data_dir,
+        port,
+        log_stdout,
         mining,
         mining_period,
         reset,
@@ -181,6 +212,8 @@ def noded(
     nodocker,
     docker_tag,
     data_dir,
+    port,
+    log_stdout,
     mining,
     mining_period,
     reset,
@@ -226,11 +259,11 @@ def noded(
         echo(f"Creating plain {node_impl}d")
         if node_impl == "bitcoin":
             my_node = BitcoindPlainController(
-                bitcoind_path=find_node_executable("bitcoin")
+                bitcoind_path=find_node_executable("bitcoin"), rpcport=port
             )
         elif node_impl == "elements":
             my_node = ElementsPlainController(
-                elementsd_path=find_node_executable("elements")
+                elementsd_path=find_node_executable("elements"), rpcport=port
             )
         Path(data_dir).mkdir(parents=True, exist_ok=True)
     else:
@@ -250,12 +283,14 @@ def noded(
                 cleanup_at_exit=True,
                 cleanup_hard=cleanuphard,
                 datadir=data_dir,
+                log_stdout=log_stdout,
             )
         elif node_impl == "elements":
             my_node.start_elementsd(
                 cleanup_at_exit=True,
                 cleanup_hard=cleanuphard,
                 datadir=data_dir,
+                log_stdout=log_stdout,
             )
     except docker.errors.ImageNotFound:
         echo(f"Image with tag {docker_tag} does not exist!")
@@ -264,9 +299,11 @@ def noded(
         )
         sys.exit(1)
     except Exception as e:
-        if e.startswith("There is already a node running!"):
+        if str(e).startswith("There is already a node running!"):
             echo(f"{e} please reset via:")
-            echo(f"python3 -m cryptoadvacne.specter {node_impl}d --reset")
+            echo(f"python3 -m cryptoadvance.specter {node_impl}d --reset")
+        else:
+            raise e
     if not nodocker:
         tags_of_image = [
             image.split(":")[-1] for image in my_node.btcd_container.image.tags
@@ -364,29 +401,46 @@ def miner_loop(node_impl, my_node, data_folder, mining_every_x_seconds, echo):
         f"height: {my_node.rpcconn.get_rpc().getblockchaininfo()['blocks']} | ",
         nl=False,
     )
+    exit = Event()
+
+    def exit_now(signo, _frame):
+        exit.set()
+
+    for sig in ("HUP", "INT"):
+        signal.signal(getattr(signal, "SIG" + sig), exit_now)
+    prevent_mining_file = Path("prevent_mining")
     i = 0
     while True:
         try:
-            my_node.mine()
             current_height = my_node.rpcconn.get_rpc().getblockchaininfo()["blocks"]
+            exit.wait(mining_every_x_seconds)
+            if not prevent_mining_file.is_file():
+                my_node.mine()
+            else:
+                echo("X", prefix=False, nl=False)
+                continue
+
+            echo("%i" % (i % 10), prefix=False, nl=False)
+            if i % 10 == 9:
+                echo(" ", prefix=False, nl=False)
+            i += 1
+            if i >= 50:
+                i = 0
+                echo("", prefix=False)
+                echo(
+                    f"height: {current_height} | ",
+                    nl=False,
+                )
+
         except Exception as e:
             logger.debug(
                 f"Caught {e}, Couldn't mine, assume SIGTERM occured => exiting!"
             )
             echo(f"THE_END(@height:{current_height})")
+            if prevent_mining_file.is_file():
+                echo("Deleting file prevent_mining")
+                prevent_mining_file.unlink()
             break
-        echo("%i" % (i % 10), prefix=False, nl=False)
-        if i % 10 == 9:
-            echo(" ", prefix=False, nl=False)
-        i += 1
-        if i >= 50:
-            i = 0
-            echo("", prefix=False)
-            echo(
-                f"height: {current_height} | ",
-                nl=False,
-            )
-        time.sleep(mining_every_x_seconds)
 
 
 def mine_2_specter_wallets(node_impl, my_node, data_folder, echo):
