@@ -49,6 +49,9 @@ class BaseConfig(object):
     SPECTER_URL_PREFIX = "/spc"
     # This enables the isolated_client-extensions, SECURITY-CRITICAL
     SESSION_COOKIE_PATH = SPECTER_URL_PREFIX
+    # should be "strong" until you're running specter behind a loadbalancer which accesses specter with more than one IP
+    SESSION_PROTECTION = os.getenv("SESSION_PROTECTION", "strong")
+    SESSION_PROTECTION = None if SESSION_PROTECTION == "none" else SESSION_PROTECTION
     # The prefix for extensions which get access to the session cookie
     EXT_URL_PREFIX = "/spc/ext"
     # The prefix for extensions which don't get access to the session cookie (if SPECTER_URL_PREFIX isn't compromised)
@@ -167,6 +170,7 @@ class BaseConfig(object):
     # List of extensions (services) to potentially load
     EXTENSION_LIST = [
         "cryptoadvance.specter.services.swan.service",
+        "cryptoadvance.specterext.liquidissuer.service",
         "cryptoadvance.specter.services.bitcoinreserve.service",
         "cryptoadvance.specterext.devhelp.service",
     ]
@@ -248,11 +252,12 @@ class CypressTestConfig(TestConfig):
 
 
 class ProductionConfig(BaseConfig):
-    SECRET_KEY = secrets.token_urlsafe(16)
+    # Injectable, as having a random SECRET-KEY won't work for gunicorn using more than one worker (might have more issues with multiple workers, though)
+    SECRET_KEY = os.getenv("SECRET_KEY", secrets.token_urlsafe(16))
     # There are some really slow machines out there. Creating a 2/4 multisig on an older MacBookAir
     # Take already >30secs
-    BITCOIN_RPC_TIMEOUT = 60
-    LIQUID_RPC_TIMEOUT = 120
+    BITCOIN_RPC_TIMEOUT = float(os.getenv("BITCOIN_RPC_TIMEOUT", "60"))
+    LIQUID_RPC_TIMEOUT = float(os.getenv("LIQUID_RPC_TIMEOUT", "120"))
 
     # Repeating it here as it's SECURITY CRITICAL. Check comments in BaseConfig
     SERVICES_LOAD_FROM_CWD = False
