@@ -1,6 +1,7 @@
 class DeviceTypes:
     BITBOX02 = "bitbox02"
     BITCOINCORE = "bitcoincore"
+    BITCOINCORE_WATCHONLY = "bitcoincore_watchonly"
     COBO = "cobo"
     COLDCARD = "coldcard"
     ELECTRUM = "electrum"
@@ -13,3 +14,4 @@ class DeviceTypes:
     SPECTERDIY = "specter"
     TREZOR = "trezor"
     KEYSTONE = "keystone"
+    PASSPORT = "passport"
