@@ -14,9 +14,16 @@ import logging
 import bitbox02
 from typing import Callable
 from flask import current_app as app
-from .helpers import deep_update, hwi_get_config, save_hwi_bridge_config
+from .helpers import (
+    deep_update,
+    hwi_get_config,
+    save_hwi_bridge_config,
+    is_testnet,
+    is_liquid,
+)
 from hwilib.devices.bitbox02 import Bitbox02Client
 from .devices.hwi.specter_diy import SpecterClient
+from .devices.hwi.jade import JadeClient
 
 logger = logging.getLogger(__name__)
 
@@ -71,7 +78,7 @@ class HWIBridge(JSONRPC):
         # Running enumerate after beginning an interaction with a specific device
         # crashes python or make HWI misbehave. For now we just get all connected
         # devices once per session and save them.
-        print("Initializing HWI...")  # to explain user why it takes so long
+        logger.info("Initializing HWI...")  # to explain user why it takes so long
         self.enumerate()
 
     @locked(hwilock)
@@ -85,7 +92,7 @@ class HWIBridge(JSONRPC):
         for devcls in hwi_classes:
             try:
                 # calling device-specific enumerate
-                if passphrase is not None:
+                if passphrase:
                     devs = devcls.enumerate(passphrase)
                 # not sure if it will handle passphrase correctly
                 # so remove it if None
@@ -100,7 +107,7 @@ class HWIBridge(JSONRPC):
                     if (
                         "needs_passphrase_sent" in dev
                         and dev["needs_passphrase_sent"]
-                        and passphrase is None
+                        and not passphrase
                     ):
                         continue
                     client = None
@@ -236,14 +243,10 @@ class HWIBridge(JSONRPC):
             # Client will be configured for testnet if our Specter instance is
             #   currently connected to testnet. This will prevent us from
             #   getting mainnet xpubs unless we set is_testnet here:
-            try:
-                client.chain = (
-                    Chain.TEST
-                    if derivation.split("/")[2].startswith("1")
-                    else Chain.MAIN
-                )
-            except:
-                client.chain = Chain.MAIN
+            der = bip32.parse_path(derivation)
+            client.chain = (
+                Chain.TEST if len(der) > 2 and der[1] == 0x80000001 else Chain.MAIN
+            )
 
             network = networks.get_network(
                 "main" if client.chain == Chain.MAIN else "test"
@@ -266,14 +269,15 @@ class HWIBridge(JSONRPC):
     @locked(hwilock)
     def display_address(
         self,
-        descriptor={},
+        descriptor="",
+        xpubs_descriptor="",
         device_type=None,
         path=None,
         fingerprint=None,
         passphrase="",
         chain="",
     ):
-        if descriptor == "":
+        if descriptor == "" and xpubs_descriptor == "":
             raise Exception("Descriptor must not be empty")
 
         with self._get_client(
@@ -283,20 +287,10 @@ class HWIBridge(JSONRPC):
             passphrase=passphrase,
             chain=chain,
         ) as client:
-            if descriptor.get("xpubs_descriptor", None):
-                try:
-                    status = hwi_commands.displayaddress(
-                        client, desc=descriptor["xpubs_descriptor"]
-                    )
-                except Exception:
-                    status = hwi_commands.displayaddress(
-                        client, desc=descriptor.get("descriptor", "")
-                    )
+            if xpubs_descriptor:
+                status = hwi_commands.displayaddress(client, desc=xpubs_descriptor)
             else:
-                status = hwi_commands.displayaddress(
-                    client, desc=descriptor.get("descriptor", "")
-                )
-            client.close()
+                status = hwi_commands.displayaddress(client, desc=descriptor)
             if "error" in status:
                 raise Exception(status["error"])
             elif "address" in status:
@@ -323,8 +317,10 @@ class HWIBridge(JSONRPC):
             passphrase=passphrase,
             chain=chain,
         ) as client:
-            if isinstance(client, SpecterClient):
-                return client.sign_b64psbt(psbt)
+            if is_liquid(chain):
+                if not hasattr(client, "sign_pset"):
+                    raise Exception("Device can't sign liquid transaction")
+                return client.sign_pset(psbt)
             status = hwi_commands.signtx(client, psbt)
             if "error" in status:
                 raise Exception(status["error"])
@@ -405,25 +401,31 @@ class HWIBridge(JSONRPC):
         device = self.detect_device(
             device_type=device_type, fingerprint=fingerprint, path=path
         )
-        if device:
-            devcls = get_device_class(device["type"])
-            if devcls:
-                client = devcls.get_client(device["path"], passphrase)
-            if not client:
-                raise Exception(
-                    "The device was identified but could not be reached.  Please check it is properly connected and try again"
-                )
-            try:
-                if chain == "liquidv1":
-                    chain = "main"
-                client.chain = Chain.argparse(chain)
-                yield client
-            finally:
-                client.close()
-        else:
+        if not device:
             raise Exception(
                 "The device could not be found. Please check it is properly connected and try again"
             )
+        devcls = get_device_class(device["type"])
+        if devcls:
+            client = devcls.get_client(device["path"], passphrase)
+        if not client:
+            raise Exception(
+                "The device was identified but could not be reached.  Please check it is properly connected and try again"
+            )
+        try:
+            if is_liquid(chain):
+                client.chain = Chain.TEST if is_testnet(chain) else Chain.MAIN
+            else:
+                client.chain = Chain.argparse(chain)
+            # hack for Jade
+            if type(client) is JadeClient:
+                if is_liquid(chain):
+                    client.set_liquid_network(chain)
+                elif chain == "signet":
+                    client.chain = Chain.TEST
+            yield client
+        finally:
+            client.close()
 
     def _extract_xpubs_from_client(self, client, account=0):
         try:
