@@ -7,9 +7,10 @@ from datetime import datetime
 from io import StringIO
 from math import isnan
 from numbers import Number
+import re
 
 import requests
-from cryptoadvance.specter.util.psbt_creator import PsbtCreator
+from cryptoadvance.specter.commands.psbt_creator import PsbtCreator
 from cryptoadvance.specter.wallet import Wallet
 from flask import Blueprint, stream_with_context
 from flask import current_app as app
@@ -29,6 +30,7 @@ from ..util.descriptor import Descriptor
 from ..util.fee_estimation import FeeEstimationResultEncoder, get_fees
 from ..util.price_providers import get_price_at
 from ..util.tx import decoderawtransaction
+from embit.descriptor.checksum import add_checksum
 
 logger = logging.getLogger(__name__)
 
@@ -65,7 +67,7 @@ def generatemnemonic():
 @login_required
 @app.csrf.exempt
 def txout_set_info():
-    res = app.specter.rpc.gettxoutsetinfo()
+    res = app.specter.rpc.gettxoutsetinfo(timeout=3600)
     return res
 
 
@@ -325,7 +327,6 @@ def rescan_progress(wallet_alias):
         )
     except SpecterError as se:
         app.logger.error("SpecterError while get wallet rescan_progress: %s" % se)
-        return {}
 
 
 @wallets_endpoint_api.route("/wallet/<wallet_alias>/get_label", methods=["POST"])
@@ -490,15 +491,35 @@ def addressinfo(wallet_alias):
         wallet = app.specter.wallet_manager.get_by_alias(wallet_alias)
         address = request.form.get("address", "")
         if address:
-            descriptor = wallet.get_descriptor(address=address)
+            descriptor = wallet.get_descriptor(
+                address=address, keep_xpubs=False, to_string=True, with_checksum=True
+            )
+            xpubs_descriptor = wallet.get_descriptor(
+                address=address, keep_xpubs=True, to_string=True, with_checksum=True
+            )
+            # The last two regex groups are optional since Electrum's derivation path is shorter
+            derivation_path_pattern = (
+                r"(\/[0-9h]+)(\/[0-9h]+)(\/[0-9h]+)(\/[0-9h]+)?(\/[0-9h]+)?"
+            )
+            # Only "descriptor" gives full derivation path, looks usually like this:
+            # wpkh([8c24a510/84h/1h/0h/0/0]0331edcb16cfd ... e02552539d984)#35zjhlhm
+            match = re.search(derivation_path_pattern, descriptor)
+            if not match:
+                logger.debug(
+                    f"Derivation path of this descriptor {descriptor} could not be parsed. Sth. wrong with the regex pattern which was {derivation_path_pattern}?"
+                )
+            logger.debug(f"This is the derivation path match: {match.group()}")
+            derivation_path = "m" + match.group()
             address_info = wallet.get_address_info(address=address)
             return {
                 "success": True,
                 "address": address,
                 "descriptor": descriptor,
+                "xpubs_descriptor": xpubs_descriptor,
+                "derivation_path": derivation_path,
                 "walletName": wallet.name,
                 "isMine": address_info and not address_info.is_external,
-                **address_info,
+                **address_info,  # address_info is an instance of Address(dict)
             }
     except Exception as e:
         handle_exception(e)
