@@ -1,10 +1,13 @@
-import requests
 import logging
-from ..specter_error import SpecterError, handle_exception
-from urllib3.exceptions import NewConnectionError
-from requests.exceptions import ConnectionError, HTTPError
 from json.decoder import JSONDecodeError
 
+import requests
+from requests.exceptions import ConnectionError, HTTPError
+from urllib3.exceptions import NewConnectionError
+
+from ..specter_error import SpecterError, handle_exception
+from ..util.requests_tools import failsafe_request_get
+
 logger = logging.getLogger(__name__)
 
 OZ_TO_G = 28.3495231
@@ -102,6 +105,8 @@ def update_price(specter, current_user):
         specter.update_alt_rate(price, current_user)
         specter.update_alt_symbol(symbol, current_user)
         return True
+    except SpecterError as e:
+        logger.error(f"{e} while updating price")
     except Exception as e:
         handle_exception(e)
         return False
@@ -188,14 +193,8 @@ def get_price_at(specter, timestamp="now"):
         raise Exception("get_price_at called whereas specter.price_check is False")
     except SpecterError as se:
         raise se
-    except (ConnectionRefusedError, ConnectionError, NewConnectionError) as e:
-        logger.error(e)
-        raise SpecterError(f"There are connection issue with the exchange:{e}")
     except KeyError as ke:
         raise SpecterError(f"Error as json doesn't look reasonable: {ke}")
-    except Exception as e:
-        handle_exception(e)
-        raise SpecterError(e)
 
 
 def _parse_exchange_currency(exchange_currency):
@@ -206,27 +205,3 @@ def _parse_exchange_currency(exchange_currency):
     elif len(arr) == 3:
         return f"{arr[0]}_{arr[1]}", arr[2]
     raise SpecterError(f"Cannot parse exchange_currency: {exchange_currency}")
-
-
-def failsafe_request_get(requests_session, url):
-    """wrapping requests which is only emitting reasonable SpecterErrors which are hopefully meaningful to the user"""
-    try:
-        response: requests.Response = requests_session.get(url)
-        if response.status_code != 200:
-            response.raise_for_status()
-        json_response = response.json()
-        if json_response.get("errors"):
-            raise SpecterError(f"JSON error: {json_response}")
-        return response.json()
-    except HTTPError as httpe:
-        try:
-            json_response = response.json()
-        except JSONDecodeError:
-            raise SpecterError(f"HttpError {httpe.response.status_code} for {url}")
-        logger.debug(f"json-response: {json_response}")
-        if json_response.get("errors"):
-            raise SpecterError(f"JSON error: {json_response}")
-        raise SpecterError(f"HttpError {httpe.response.status_code} for {url}")
-    except Exception as e:
-        handle_exception(e)
-        raise SpecterError(e)
